/**
 * @namespace The PrimeFaces Extensions root namespace.
 */
PrimeFacesExt = {
    /**
     * Gets the extension of the current request URL.
     *
     * @author Thomas Andraschko
     * @returns {string} The URL extensions.
     */
	getRequestUrlExtension : function() {
		return PrimeFacesExt.getUrlExtension(location.href);
	},	
		
    /**
     * Gets the extension of the given URL.
     *
     * @author Thomas Andraschko
     * @returns {string} The URL extensions.
     */
	getUrlExtension : function(url) {
		return (url = url.substr(1 + url.lastIndexOf("/")).split('?')[0]).substr(url.lastIndexOf("."));
	},

    /**
     * Builds a resource URL for given parameters.
     * 
     * @author Thomas Andraschko
     * @param {string} name The name of the resource. For example: /core/core.js
     * @param {string} library The library of the resource. For example: primefaces-extensions
     * @param {string} version The version of the library. For example: 0.2.0-SNAPSHOT
     * @returns {string} The resource URL.
     */
	getFacesResource : function(name, library, version) {
		var scriptURI = PrimeFacesExt.getPrimeFacesExtensionsScriptURI();
        
		scriptURI = scriptURI.replace('/primefaces-extensions.js', name);
		
		if (PrimeFacesExt.useUncompressedResources) {
			scriptURI = scriptURI.replace('ln=' + PrimeFacesExt.RESOURCE_LIBRARY_UNCOMPRESSED, 'ln=' + library);
		} else {
			scriptURI = scriptURI.replace('ln=' + PrimeFacesExt.RESOURCE_LIBRARY, 'ln=' + library);
		}

		var extractedVersion = RegExp('[?&]v=([^&]*)').exec(scriptURI)[1];
		if (version) {
			scriptURI = scriptURI.replace('v=' + extractedVersion, 'v=' + version);
		} else {
			scriptURI = scriptURI.replace('v=' + extractedVersion, '');
		}

        return location.protocol + '//' + location.host + scriptURI;
	},

    /**
     * Gets the version of the current PrimeFaces Extensions library.
     *
     * @author Thomas Andraschko
     * @returns {string} The PrimeFaces Extensions version.
     */
	getPrimeFacesExtensionsVersion : function() {
		if (!PrimeFacesExt.VERSION) {
			var scriptURI = PrimeFacesExt.getPrimeFacesExtensionsScriptURI();
			PrimeFacesExt.VERSION = RegExp('[?&]v=([^&]*)').exec(scriptURI)[1];
		}

		return PrimeFacesExt.VERSION;
	},

    /**
     * Builds a resource URL for a PrimeFaces Extensions resource.
     *
     * @author Thomas Andraschko
     * @returns {string} The resource URL.
     */
	getPrimeFacesExtensionsResource : function(name) {
		var resourceLibrary = PrimeFacesExt.RESOURCE_LIBRARY;
		if (PrimeFacesExt.useUncompressedResources) {
			resourceLibrary = PrimeFacesExt.RESOURCE_LIBRARY_UNCOMPRESSED;
		}

		return PrimeFacesExt.getFacesResource(
				name,
				resourceLibrary,
				PrimeFacesExt.getPrimeFacesExtensionsVersion());
	},

    /**
     * Builds a resource URL for a PrimeFaces Extensions Compressed resource.
     * 
     * @author Thomas Andraschko
     * @param {string} name The name of the resource. For example: /core/core.js
     * @returns {string} The resource URL.
     */
	getPrimeFacesExtensionsCompressedResource : function(name) {
		return PrimeFacesExt.getFacesResource(
				name,
				PrimeFacesExt.RESOURCE_LIBRARY,
				PrimeFacesExt.getPrimeFacesExtensionsVersion());
	},

    /**
     * Checks if the FacesServlet is mapped with extension mapping. For example: .jsf/.xhtml.
     *
     * @author Thomas Andraschko
     * @returns {boolean} If mapped with extension mapping.
     */
	isExtensionMapping : function() {
		if (!PrimeFacesExt.IS_EXTENSION_MAPPING) {
			var scriptURI = PrimeFacesExt.getPrimeFacesExtensionsScriptURI();
			var primeFacesExtensionsScript = 'primefaces-extensions.js';

			PrimeFacesExt.IS_EXTENSION_MAPPING = scriptURI.charAt(scriptURI.indexOf(primeFacesExtensionsScript) + primeFacesExtensionsScript.length) === '.';
		}

		return PrimeFacesExt.IS_EXTENSION_MAPPING;
	},

    /**
     * Checks if the current included scripts are uncompressed.
     *
     * @author Thomas Andraschko
     * @returns {boolean} If uncompresed resources are used.
     */
	useUncompressedResources : function() {
		if (!PrimeFacesExt.USE_UNCOMPRESSED_RESOURCES) {
			var scriptURI = PrimeFacesExt.getPrimeFacesExtensionsScriptURI();

			PrimeFacesExt.USE_UNCOMPRESSED_RESOURCES = scriptURI.indexOf(RESOURCE_LIBRARY_UNCOMPRESSED) !== -1;
		}

		return PrimeFacesExt.USE_UNCOMPRESSED_RESOURCES;
	},

    /**
     * Gets the resource URI of the current included primefaces-extensions.js.
     *
     * @author Thomas Andraschko
     * @returns {string} The resource URI.
     */
	getPrimeFacesExtensionsScriptURI : function() {
		if (!PrimeFacesExt.SCRIPT_URI) {
			PrimeFacesExt.SCRIPT_URI = $('script[src*="' + PrimeFacesExt.RESOURCE_IDENTIFIER + '/primefaces-extensions.js"]').attr('src');
		}

		return PrimeFacesExt.SCRIPT_URI;
	},

	/**
	 * Load a JavaScript file from the server using a GET HTTP request, then execute it.
	 * 
	 * @author Thomas Andraschko
	 * @param {string} url A string containing the URL to which the request is sent.
	 * @param {function} callback A callback function that is executed if the request succeeds.
	 * @param {string} cache Appends a unique timestamp if false.
	 */
	getScript : function(url, callback, cache) {
		$.ajax({
				type: "GET",
				url: url,
				success: callback,
				dataType: "script",
				cache: cache
		});
	},

	/**
	 * Creates a widget and load the required resources if not already available.
	 * The .js and .css must has the same name as the widget and must be placed inside a directory with also the name.
	 * The file and directory names must be completely in lower case.
	 * For example: /imageareaselect/imageareaselect.js.
	 * 
	 * @author Thomas Andraschko
	 * @param {string} widgetName The name of the widget. For example: ImageAreaSelect.
	 * @param {widgetVar} widgetVar The variable in the window object for accessing the widget.
	 * @param {object} cfg An object with options.
	 * @param {boolean} hasStyleSheet If the css file should be loaded as well.
	 */
	cw : function(widgetName, widgetVar, cfg, hasStyleSheet) {
	    PrimeFacesExt.createWidget(widgetName, widgetVar, cfg, hasStyleSheet);
	},

	/**
	 * Creates a widget and load the required resources if not already available.
	 * The .js and .css must has the same name as the widget and must be placed inside a directory with also the name.
	 * The file and directory names must be completely in lower case.
	 * For example: /imageareaselect/imageareaselect.js.
	 * 
	 * @author Thomas Andraschko
	 * @param {string} widgetName The name of the widget. For example: ImageAreaSelect.
	 * @param {widgetVar} widgetVar The variable in the window object for accessing the widget.
	 * @param {object} cfg An object with options.
	 * @param {boolean} hasStyleSheet If the css file should be loaded as well.
	 */
	createWidget : function(widgetName, widgetVar, cfg, hasStyleSheet) {            
	    if (PrimeFacesExt.widget[widgetName]) {
	    	PrimeFacesExt.initWidget(widgetName, widgetVar, cfg);
	    } else {
	    	if (hasStyleSheet === true) {
	    		var styleSheet =
	    			PrimeFacesExt.getPrimeFacesExtensionsResource('/' + widgetName.toLowerCase() + '/' + widgetName.toLowerCase() + '.css');

	    		//insert style sheet after last style sheet with ln=primefaces
	    		if ($.browser.msie) {
	    			var indexToInsert;
	    			for (var i = 0; i < document.styleSheets.length; i++ ) {
	    				var currentStyleSheetURL = document.styleSheets[i].href.toString();
	    			    if (currentStyleSheetURL.indexOf('ln=primefaces') !== -1) {
	    			    	//add +1 to insert after this style sheet
	    			    	indexToInsert = i + 1;
	    			    }
	    			}

	    			if (indexToInsert) {
	    				document.createStyleSheet(styleSheet, indexToInsert);
	    			} else {
	    				PrimeFaces.error('No style sheet from PrimeFaces or PrimeFaces Extensions included. StyleSheet for PrimeFaces Extensions Widget ' + widgetName + ' will not be added.');
	    			}
	    		} else {
		    		var lastStyleSheet = $('link[href*="ln=primefaces"]:last');

		    		if (lastStyleSheet.length > 0) {
		    			lastStyleSheet.after('<link type="text/css" rel="stylesheet" href="' + styleSheet + '" />');
		    		} else {
		    			PrimeFaces.error('No style sheet from PrimeFaces or PrimeFaces Extensions included. StyleSheet for PrimeFaces Extensions Widget ' + widgetName + ' will not be added.');
		    		}
	    		}
	    	}

    		var script =
    			PrimeFacesExt.getPrimeFacesExtensionsResource('/' + widgetName.toLowerCase() + '/' + widgetName.toLowerCase() + '.js');

    		//load script
	        PrimeFacesExt.getScript(script, function() {
	        	PrimeFacesExt.initWidget(widgetName, widgetVar, cfg);
	        }, true);
	    }
	},

	/**
	 * Creates the widget or calls "refresh" if already available.
	 * 
	 * @author Thomas Andraschko
	 * @param {string} widgetName The name of the widget. For example: ImageAreaSelect.
	 * @param {widgetVar} widgetVar The variable in the window object for accessing the widget.
	 * @param {object} cfg An object with options.
	 */
	initWidget : function(widgetName, widgetVar, cfg) {
		if (window[widgetVar]) {
			window[widgetVar].refresh(cfg);
		} else {
			window[widgetVar] = new PrimeFacesExt.widget[widgetName](cfg);
		}
	},

	/**
	 * Configures component specific localized text by given widget name and locale in configuration object.
	 * 
	 * @author Oleg Varaksin
	 * @param {string} widgetName The name of the widget. For example: 'TimePicker'.
	 * @param {object} cfg Configuration object as key, value pair. This object should keep current locale in cfg.locale.
     * @returns {object} cfg Configuration object with updated localized text (if any text to given locale were found). 
	 */    
    configureLocale : function(widgetName, cfg) {
        var localeSettings = PrimeFacesExt.locales[widgetName][cfg.locale];
        if(localeSettings) {
            for(var setting in localeSettings) {
                cfg[setting] = localeSettings[setting];
            }
        }
        
        return cfg;
    },

	/**
	 * Removes the widget's script block from the DOM.
	 * 
	 * @author Thomas Andraschko
	 * @param {string} clientId The id of the widget.
	 */
	removeWidgetScript : function(clientId) {
		$(PrimeFaces.escapeClientId(clientId) + '_s').remove();
	},

	/**
	 * The JSF resource identifier.
	 * 
	 * @author Thomas Andraschko
	 * @type {string}
	 * @constant
	 */
	RESOURCE_IDENTIFIER : '/javax.faces.resource',

	/**
	 * The name of the PrimeFaces Extensions resource library.
	 *
	 * @author Thomas Andraschko
	 * @type {string}
	 * @constant
	 */
	RESOURCE_LIBRARY : 'primefaces-extensions',

	/**
	 * The name of the PrimeFaces Extensions Uncompressed resource library.
	 *
	 * @author Thomas Andraschko
	 * @type {string}
	 * @constant
	 */
	RESOURCE_LIBRARY_UNCOMPRESSED : 'primefaces-extensions-uncompressed'
};

/**
 * @namespace Namespace for behaviors.
 */
PrimeFacesExt.behavior = {};

/**
 * @namespace Namespace for widgets.
 */
PrimeFacesExt.widget = {};

/**
 * @namespace Namespace for localization.
 */
PrimeFacesExt.locales = {};

/**
 * @namespace Namespace for TimePicker localization.
 */
PrimeFacesExt.locales.TimePicker = {};

/**
 * JavaScript behavior.
 * 
 * @author Thomas Andraschko
 * @constructor
 */
PrimeFacesExt.behavior.Javascript = function(cfg, ext) {
	return cfg.execute.call(this, cfg.source, cfg.event, cfg.params, ext);
};
;PrimeFacesExt.getAjaxErrorHandlerInstance = function() {
	if (!PrimeFacesExt.AJAX_ERROR_HANDLER_INSTANCE) {
		var instance = new PrimeFacesExt.widget.AjaxErrorHandler();

		//  INIT IS CALLED AUTOMATICALLY ... instance.init();
		
		PrimeFacesExt.AJAX_ERROR_HANDLER_INSTANCE = instance;
	}

	return PrimeFacesExt.AJAX_ERROR_HANDLER_INSTANCE;
}

/**
 * PrimeFaces Extensions AjaxErrorHandler.
 *
 * @author Pavol Slany
 */
PrimeFacesExt.widget.AjaxErrorHandler = PrimeFaces.widget.BaseWidget.extend({
	
	DEFAULT_HOSTNAME : '???unknown???',

	init : function() {
		this.popupWindow = null;
		this.popupWindowRoot = null;
		this.popupWindowMask = null;

		this.hostname = this.DEFAULT_HOSTNAME;
		
		this.defaultSettings = {
			'title': '{error-name}',
			'body': '{error-message}',
			'button': 'Reload',
			'buttonOnclick': function() {
				document.location.href = document.location.href;
			},
			'onerror': function(error, response) {
				
			}
		};
		this.settings = this.defaultSettings;
		this.otherSettings = {};

		this.overwritePrimeFacesAjaxResponse();
	},

	overwritePrimeFacesAjaxResponse : function() {
		var _self = this;

		// backup original AjaxResponse function ...
		var backupAjaxResponse = PrimeFaces.ajax.AjaxResponse;

		PrimeFaces.ajax.AjaxResponse = function() {
			var docPartialUpdate = arguments[0];
			var nodeErrors = docPartialUpdate.getElementsByTagName('error');

			if (nodeErrors && nodeErrors.length && nodeErrors[0].childNodes && nodeErrors[0].childNodes.length) {
				// XML => JSON
				var error = {};

				for (var i=0; i < nodeErrors[0].childNodes.length; i++) {
					var node = nodeErrors[0].childNodes[i];
					var key = node.nodeName;
					var val = node.nodeValue;

					if (node.childNodes && node.childNodes.length) {
						val = node.childNodes[0].nodeValue;
					}

					error[key] = val;
				}

				if (error['error-name']) {
					// findErrorSettings
					var errorSetting = _self.findErrorSettings(error['error-name']);

					//skip dialog if onerror is defined and returns false
					if (errorSetting['onerror']) {
						var onerrorFunction = errorSetting['onerror'];
						if (onerrorFunction.call(this, error, arguments[2]) === false) {
							return true;
						}
					}

					// Copy updates to errorSettings ...
					if (error.updateCustomContent && error.updateCustomContent.substring(-13) == '<exception />') {
						error.updateCustomContent = null;
					}

					if (error.updateTitle && error.updateTitle.substring(-13) == '<exception />') {
						error.updateTitle = null;
					}

					if (error.updateBody && error.updateBody.substring(-13) == '<exception />') {
						error.updateBody = null;
					}

					if (error.updateViewState && error.updateViewState.substring(-13) == '<exception />') {
						error.updateViewState = null;
					}

					errorSetting.updateCustomContent = error.updateCustomContent;
					errorSetting.updateTitle = error.updateTitle;
					errorSetting.updateBody = error.updateBody;
					errorSetting.updateViewState = error.updateViewState;

					var errorData = _self.replaceVariables(errorSetting, error);

					_self.show(errorData);

					return true;
				}
			}

			return backupAjaxResponse.apply(this, arguments);
		};
	},
	
	isVisible : function() {
		return this.popupWindow && this.popupWindow.isVisible();
	},
	
	hide : function () {
		if (this.dialogWidget) {
			this.dialogWidget.hide();
			this.dialogWidget = null;

			this.dialog.remove();
		}
	},
	
	show : function(errorData) {
		this.hide();

		//create required html
		this.dialog = $('<div id="ajaxErrorHandlerDialog" class="ui-dialog ui-widget ui-widget-content ui-overlay-hidden ui-corner-all ui-shadow pe-ajax-error-handler" style="width: auto; height: auto;"></div>');

		//append to DOM
		$('body').append(this.dialog);

		//custom content?
		if (errorData.updateCustomContent) {
			this.dialog.append(errorData.updateCustomContent);
		} else {
			//create required html
			var dialogHeader = $('<div class="ui-dialog-titlebar ui-widget-header ui-helper-clearfix ui-corner-top"></div>');
			var dialogHeaderText = $('<span class="ui-dialog-title"></span>');
			var dialogContent = $('<div class="ui-dialog-content ui-widget-content" style="height: auto;"></div>');
			var dialogButtonPane = $('<div class="ui-dialog-buttonpane ui-widget-content ui-helper-clearfix"></div>')
			
			//append to DOM
			dialogHeader.append(dialogHeaderText);

			this.dialog.append(dialogHeader);
			this.dialog.append(dialogContent);
			this.dialog.append(dialogButtonPane);

			//setup button
			var dialogButton = $('<button class="ui-button ui-widget ui-state-default ui-corner-all ui-button-text-only"><span class="ui-button-text">' + errorData['button'] + '</span></button>');
			var buttonOnclickFunction = errorData['buttonOnclick'];
			dialogButton.click(function() {
				buttonOnclickFunction.call(this);
			});
			
			dialogButtonPane.append(dialogButton);

			//add title
			if (errorData.updateTitle) {
				dialogHeaderText.append(errorData.updateTitle);
			} else {
				dialogHeaderText.append(errorData.title);
			}

			//add body
			if (errorData.updateBody) {
				dialogContent.append(errorData.updateBody);
			} else {
				dialogContent.append(errorData.body);
			}
		}

		//create Dialog widget
		this.dialogWidget = new PrimeFaces.widget.Dialog({
			id : 'ajaxErrorHandlerDialog',
			resizable : true,
			draggable : true,
			modal : true
		});

		this.dialogWidget.show();
	},

	getDefaultErrorTime : function() {
		return new Date().toString();
	},
	
	findErrorSettings : function(name) {
		if (!name) {
			return jQuery.extend({}, this.settings);
		}

		if (!this.otherSettings[name]) {
			jQuery.extend({}, this.settings);
		}

		return jQuery.extend({}, this.settings, this.otherSettings[name]);
	},

	addErrorSettings : function(newSettings) {
		if (!newSettings) {
			return;
		}

		if (!newSettings.type) {
			this.settings = jQuery.extend({}, this.settings, newSettings);
			return;
		}

		var type = this.otherSettings[newSettings.type] || {};

		this.otherSettings[newSettings.type] = jQuery.extend({}, type, newSettings);
	},
	
	replaceAll : function(str, key, val) {
		var newStr;

		while ((newStr = str.replace(key, val)) != str) {
			str = newStr
		};

		return str;
	},
	
	replaceVariables : function(obj, variables) {
		if (!obj) {
			return text;
		}

		variables = jQuery.extend({
			'error-hostname': this.hostname,
			'error-stacktrace': '',
			'error-time': this.getDefaultErrorTime()
		}, variables);

		var returnValue = {};

		jQuery.each(obj, $.proxy(function(key, val) {
			if (typeof(val) == 'string') {
				jQuery.each(variables, $.proxy(function(iVar, iVal) {
					val = this.replaceAll(val, '{' + iVar + '}', iVal);
				}, this));
			}

			returnValue[key] = val;
		}, this));

		return returnValue;
	},
	
	setHostname : function(hostname) {
		this.hostname = hostname;
	}
});
;/**
 * PrimeFaces Extensions AjaxStatus Widget.
 *
 * @author Oleg Varaksin
 */
PrimeFacesExt.widget.AjaxStatus = PrimeFaces.widget.BaseWidget.extend({
	
	/**
	 * Initializes the widget.
	 * 
	 * @param {object} cfg The widget configuration.
	 */
	init : function(cfg) {
	    this._super(cfg);
	},


	bindFacet : function(eventName, facetToShow) {
	    var _self = this;
	
	    // jQuery
	    jQuery(document).bind(eventName, function() {
	        _self.showFacet(facetToShow);
	    });
	
	    // JSF standard
	    if (eventName == "ajaxError") {
	        jsf.ajax.addOnError(this.processAjaxOnErrorFacet(facetToShow));
	    } else {
	        jsf.ajax.addOnEvent(this.processAjaxOnEventFacet(eventName, facetToShow));
	    }    
	},

	bindCallback : function(eventName, fn) {
	    // jQuery
	    jQuery(document).bind(eventName, fn);
	
	    // JSF standard
	    if (eventName == "ajaxError") {
	        jsf.ajax.addOnError(this.processAjaxOnErrorCallback(fn));
	    } else {
	        jsf.ajax.addOnEvent(this.processAjaxOnEventCallback(eventName, fn));
	    }
	},

	processAjaxOnEventFacet : function(eventName, facetToShow) {
	    var _self = this;
	
	    function processEvent(data) {
	        if (eventName == "ajaxStart" && data.status == "begin") {
	            _self.showFacet(facetToShow);
	        } else if (eventName == "ajaxComplete" && data.status == "complete") {
	            _self.showFacet(facetToShow);
	        } else if (eventName == "ajaxSuccess" && data.status == "success") {
	            _self.showFacet(facetToShow);
	        }
	    }
	
	    return processEvent;
	},

	processAjaxOnErrorFacet : function(facetToShow) {
	    var _self = this;
	
	    function processEvent() {
	        _self.showFacet(facetToShow);
	    }
	
	    return processEvent;
	},

	processAjaxOnEventCallback : function(eventName, fn) {
	    function processEvent(data) {
	        if (eventName == "ajaxStart" && data.status == "begin") {
	            fn();
	        } else if (eventName == "ajaxComplete" && data.status == "complete") {
	            fn();
	        } else if (eventName == "ajaxSuccess" && data.status == "success") {
	            fn();
	        }
	    }
	
	    return processEvent;
	},

	processAjaxOnErrorCallback : function(fn) {
	    function processEvent() {
	        fn();
	    }
	
	    return processEvent;
	},

	showFacet : function(facetToShow) {
	    this.jq.children().hide();
	    jQuery(this.jqId + '_' + facetToShow).show();
	}
});
;
/**
 * PrimeFaces Extensions BlockPanel Widget.
 *
 * @author Pavol Slany
 */
PrimeFacesExt.widget.BlockPanel = PrimeFaces.widget.BaseWidget.extend({

	/**
	 * Initializes the widget.
	 * 
	 * @param {object} cfg The widget configuration.
	 */
	init : function(cfg) {
		this.id = cfg.id;
		this.blocked = cfg.blocked;
		
		////////////////////////
		// Mask 
		this.getMask = function() {
			var mask = new PrimeFacesExt.widget.BlockPanel.MaskAround(this.id);
			return (this.getMask = function() {return mask})();
		}

		////////////////////////
		// FOCUS LOST bindings ...
		this.getFocusArea = function() {
			var area = new PrimeFacesExt.widget.BlockPanel.FocusArea(this.id);
			return (this.getFocusArea = function() {return area})();
		}
		
		this.block = function() {
			this.blocked = true;
	        // Show MASK
			this.getMask().show();
			
			// focus area ...
			this.getFocusArea().activate();
		}
		this.unblock = function() {
			this.blocked = false;
	        // Hide MASK
			this.getMask().hide();
			
			// focus area ...
			this.getFocusArea().deactivate()
		}
		
		if(this.blocked) {
			this.block();
		}
		else {
			this.unblock();
		}
		
        PrimeFacesExt.removeWidgetScript(this.id);
    },
    
	block : function () {
		this.block();
    },
	    
	unblock : function () {
		this.unblock();
    }
});

/**
 * @author Pavol Slany
 */
PrimeFacesExt.widget.BlockPanel.FocusArea = function(elementId) {
	var elPanel = $(PrimeFaces.escapeClientId(elementId)); 
	
	// If panel has not focusable elements => create hidden focusable element
	// Add hidden focusable HTML element
	if (!$(PrimeFaces.escapeClientId(elementId)+' :focusable').length)
		elPanel.append($('<a href="" style="width:0px;height:0px;position:absolute;"></a>'));
	
	var focusFirstElement = function() {
		$($(PrimeFaces.escapeClientId(elementId)+' :focusable')[0]).focus();
		isIn = 0;
	}
	
	var activate = false;
	
	var isIn = 0;
	var focusOutTimeout = null;
	var focusOutTimeoutHandler = function() {
		if (isIn < 0 && activate) {
			focusFirstElement();
		}
	};
	var focusin = function() {
		isIn=isIn+1;
		
		if (focusOutTimeout != null) clearTimeout(focusOutTimeout);
		focusOutTimeout = setTimeout(focusOutTimeoutHandler, 20);
	};
	var focusout = function() {
		isIn=isIn-1;
		
		if (focusOutTimeout != null) clearTimeout(focusOutTimeout);
		focusOutTimeout = setTimeout(focusOutTimeoutHandler, 20);
	};
	
	
	return {
		activate: function() {
			if (!activate) {
				elPanel.bind('focusin',focusin);
				elPanel.bind('focusout',focusout);

				focusFirstElement();
			}
			activate = true;
		}
		,
		deactivate: function() {
			if (activate) {
				elPanel.unbind('focusin',focusin);
				elPanel.unbind('focusout',focusout);
			}
			activate = false;
		}
	}
};

/**
 * @author Pavol Slany
 */
PrimeFacesExt.widget.BlockPanel.MaskAround = function(elementId) {
	var maskId = elementId+'_maskAround';

	var destinationOpacity = function() {
		var el = $('<div class="ui-widget-overlay"></div>');
		$('body').append(el);
		var opacity = el.css('opacity');
		el.remove();
		return opacity;
	}();

	var ElementPieceOfMask = function(maskKey) {
		var idEl = maskId + maskKey;
		var elementMustBeVisible = $(PrimeFaces.escapeClientId(idEl)).is(':visible');
		var getMaskElement = function() {
			var maskElement = $(PrimeFaces.escapeClientId(idEl));
			
			if (!maskElement || !maskElement.length) {
				maskElement = $('<div id="'+idEl+'" />');
				maskElement.css({
//					position: 'fixed',
                    position: 'absolute',
					top: 0,
					left: 0,
					display: 'none',
					zIndex : 999999,
					overflow: 'hidden'
				});
				maskElement.append($('<div class="ui-widget-overlay" style="position:absolute;"></div>').css('opacity', 1));
				$('body').append(maskElement);
			}
			return maskElement;
		}
		
		var updateVisibility = function() {
            var jidEl = $(PrimeFaces.escapeClientId(idEl));
			if (elementMustBeVisible) {
				if (!jidEl.is(':visible'))
					getMaskElement().fadeTo("fast", destinationOpacity, updateVisibility);
				return;
			}
			// ...
			if (jidEl.is(':visible')) {
				getMaskElement().fadeOut('fast', updateVisibility);
			}
		}

		return {
			updatePosition: function(x0,y0,x1,y1) {
				$('<div class="ui-widget-overlay"></div>')
				var el = getMaskElement();
				el.css({
					left: x0,
					top: y0,
					width: (x1-x0),
					height: (y1-y0)
				});
				
				var maxSize = getMaxSize();
				$(el.children()[0]).css({
					left: (0-x0),
					top: (0-y0),
					height: maxSize.height,
					width: maxSize.width
				});
			},
			show: function() {
				elementMustBeVisible = true;
				updateVisibility();
			},
			hide: function() {
				elementMustBeVisible = false;
				updateVisibility();
			}
		};
	};
	
	var top 	= new ElementPieceOfMask('_top');
	var left 	= new ElementPieceOfMask('_left');
	var bottom 	= new ElementPieceOfMask('_bottom');
	var right 	= new ElementPieceOfMask('_right');
	
	var getMaxSize = function() {
		var winWidth = $(window).width();
		var winHeight = $(window).height();
		var docWidth = $(document).width();
		var docHeight = $(document).height();
		
		var maxWidth = winWidth > docWidth ? winWidth : docWidth;
		var maxHeight = winHeight > docHeight ? winHeight : docHeight;
		
		return {
			width: maxWidth, 
			height: maxHeight
		};
	}

    // check IE8 browser (it works in all BROWSER MODEs and DOCUMENT MODEs)
    var isIE8 = function() {
        if ($.browser.msie) {
            // document.documentMode is since IE8
            // window.performance is since IE9
            if (document.documentMode && !window.performance) return true;
        }
        return false;
    }


    var updateMaskPositions = function() {
        var maxSize = getMaxSize();
		
		var maxWidth = maxSize.width;
		var maxHeight = maxSize.height;
		
		// Check PANEL position for MASK
		var el = $(PrimeFaces.escapeClientId(elementId));
		var x0 = el.offset().left;
		var y0 = el.offset().top;
		var x1 = x0 + el.outerWidth();
		var y1 = y0 + el.outerHeight();

        // Correct MASK position before, any parents is with overflow=AUTO|HIDDEN|SCROLL
        var elParent = el.parent();
        while (elParent.length > 0 && elParent[0].tagName!='HTML') {
            var overflow = elParent.css('overflow');

            if (overflow == 'auto' || overflow == 'hidden' || overflow == 'scroll') {
                // IE BUG - if height is 0 => CSS problem with overflow => ignore it
                if (elParent.height()>0) {
                    var offset = elParent.offset();
                    if (x0 < offset.left) x0 = offset.left;
                    if (y0 < offset.top) y0 = offset.top;
                    if (x1 > offset.left + elParent.outerWidth()) x1 = offset.left + elParent.outerWidth();
                    if (y1 > offset.top + elParent.outerHeight()) y1 = offset.top + elParent.outerHeight();
                }
            }

            elParent = elParent.parent();
        }


		if (x0<0) x0 = 0;
		if (y0<0) y0 = 0;
		if (x1<x0) x1=x0;
		if (y1<y0) y1=y0;

        if (el.outerHeight()>0 && y1-y0<=5) {
            try {
                var elFocus = $(PrimeFaces.escapeClientId(elementId)+' :focusable');
                // Change focus ...
                if (elFocus.length < 2) {
                    // If ELEMENT does not exist => create TMP element
                    var tmpEl = $('<a href="#"> </a>');
                    el.append(tmpEl);
                    tmpEl.focus();
                    tmpEl.remove();
                }
                else {
                    // ELEMENT exists
                    $(elFocus[1]).focus();
                }
                // SET FOCUS for first element
                $(elFocus[0]).focus();

            } catch(e) {}
        }

        var ie8Corecting = 0;
        // IE8 has bug with layouts => check IE8 (it works in all BROWSER MODE and DOCUMENT MODE)
        if (isIE8()) {
            ie8Corecting = 1;
        }
		
//		var bodyOffset = {top: $(window).scrollTop(), left: $(window).scrollLeft()};
        var bodyOffset = {top: 0, left: 0};

        top.updatePosition(
            0-bodyOffset.left,
            0-bodyOffset.top,
            maxWidth-bodyOffset.left,
            y0-bodyOffset.top);
        bottom.updatePosition(
            0-bodyOffset.left,
            y1-bodyOffset.top,
            maxWidth-bodyOffset.left,
            maxHeight-bodyOffset.top);
        left.updatePosition(
            0-bodyOffset.left,
            y0-bodyOffset.top + ie8Corecting,
            x0-bodyOffset.left,
            y1-bodyOffset.top - ie8Corecting);
        right.updatePosition(
            x1-bodyOffset.left,
            y0-bodyOffset.top + ie8Corecting,
            maxWidth-bodyOffset.left,
            y1-bodyOffset.top - ie8Corecting);
    }
	
	var resizeTimer = null;
	$(window).bind('resize', function() {
		if (resizeTimer) clearTimeout(resizeTimer);
		resizeTimer = setTimeout(updateMaskPositions, 100);
	});

	var updatePositions = function() {
		updateMaskPositions();
		if (mustBeShowed) 
			setTimeout(updatePositions, 150);
	};
	var mustBeShowed = false;
	var showAreas = function() {
		mustBeShowed = true;
		
		updatePositions();
		
		top.show();
		bottom.show();
		left.show();
		right.show();
	}
	var hideAreas = function() {
		mustBeShowed = false;
		
		top.hide();
		bottom.hide();
		left.hide();
		right.hide();
	}
	
	return {
		show: function() {
			showAreas();
		},
		hide: function() {
			hideAreas();
		}
	};
};;/*!
 * jQuery blockUI plugin
 * Version 2.39 (23-MAY-2011)
 * @requires jQuery v1.2.3 or later
 *
 * Examples at: http://malsup.com/jquery/block/
 * Copyright (c) 2007-2010 M. Alsup
 * Dual licensed under the MIT and GPL licenses:
 * http://www.opensource.org/licenses/mit-license.php
 * http://www.gnu.org/licenses/gpl.html
 *
 * Thanks to Amir-Hossein Sobhi for some excellent contributions!
 */

;(function($) {

if (/1\.(0|1|2)\.(0|1|2)/.test($.fn.jquery) || /^1.1/.test($.fn.jquery)) {
	alert('blockUI requires jQuery v1.2.3 or later!  You are using v' + $.fn.jquery);
	return;
}

$.fn._fadeIn = $.fn.fadeIn;

var noOp = function() {};

// this bit is to ensure we don't call setExpression when we shouldn't (with extra muscle to handle
// retarded userAgent strings on Vista)
var mode = document.documentMode || 0;
var setExpr = $.browser.msie && (($.browser.version < 8 && !mode) || mode < 8);
var ie6 = $.browser.msie && /MSIE 6.0/.test(navigator.userAgent) && !mode;

// global $ methods for blocking/unblocking the entire page
$.blockUI   = function(opts) { install(window, opts); };
$.unblockUI = function(opts) { remove(window, opts); };

// convenience method for quick growl-like notifications  (http://www.google.com/search?q=growl)
$.growlUI = function(title, message, timeout, onClose) {
	var $m = $('<div class="growlUI"></div>');
	if (title) $m.append('<h1>'+title+'</h1>');
	if (message) $m.append('<h2>'+message+'</h2>');
	if (timeout == undefined) timeout = 3000;
	$.blockUI({
		message: $m, fadeIn: 700, fadeOut: 1000, centerY: false,
		timeout: timeout, showOverlay: false,
		onUnblock: onClose, 
		css: $.blockUI.defaults.growlCSS
	});
};

// plugin method for blocking element content
$.fn.block = function(opts) {
	return this.unblock({ fadeOut: 0 }).each(function() {
		if ($.css(this,'position') == 'static')
			this.style.position = 'relative';
		if ($.browser.msie)
			this.style.zoom = 1; // force 'hasLayout'
		install(this, opts);
	});
};

// plugin method for unblocking element content
$.fn.unblock = function(opts) {
	return this.each(function() {
		remove(this, opts);
	});
};

$.blockUI.version = 2.39; // 2nd generation blocking at no extra cost!

// override these in your code to change the default behavior and style
$.blockUI.defaults = {
	// message displayed when blocking (use null for no message)
	message:  '<h1>Please wait...</h1>',

	title: null,	  // title string; only used when theme == true
	draggable: true,  // only used when theme == true (requires jquery-ui.js to be loaded)
	
	theme: false, // set to true to use with jQuery UI themes
	
	// styles for the message when blocking; if you wish to disable
	// these and use an external stylesheet then do this in your code:
	// $.blockUI.defaults.css = {};
	css: {
		padding:	0,
		margin:		0,
		width:		'auto',
		top:		'40%',
		left:		'35%',
		textAlign:	'center',
		color:		'#000',
		border:		'3px solid #aaa',
		backgroundColor:'#fff',
		cursor:		'wait'
	},
	
	// minimal style set used when themes are used
	themedCSS: {
		width:	'auto',
		top:	'40%',
		left:	'35%'
	},

	// styles for the overlay
	overlayCSS:  {
		/*backgroundColor: '#000',*/
		/*opacity:	  	 0.6,*/
		cursor:		  	 'wait'
	},

	// styles applied when using $.growlUI
	growlCSS: {
		width:  	'350px',
		top:		'10px',
		left:   	'',
		right:  	'10px',
		border: 	'none',
		padding:	'5px',
		opacity:	0.6,
		cursor: 	'default',
		color:		'#fff',
		backgroundColor: '#000',
		'-webkit-border-radius': '10px',
		'-moz-border-radius':	 '10px',
		'border-radius': 		 '10px'
	},
	
	// IE issues: 'about:blank' fails on HTTPS and javascript:false is s-l-o-w
	// (hat tip to Jorge H. N. de Vasconcelos)
	iframeSrc: /^https/i.test(window.location.href || '') ? 'javascript:false' : 'about:blank',

	// force usage of iframe in non-IE browsers (handy for blocking applets)
	forceIframe: false,

	// z-index for the blocking overlay
	baseZ: 1000,

	// set these to true to have the message automatically centered
	centerX: true, // <-- only effects element blocking (page block controlled via css above)
	centerY: true,

	// allow body element to be stetched in ie6; this makes blocking look better
	// on "short" pages.  disable if you wish to prevent changes to the body height
	allowBodyStretch: true,

	// enable if you want key and mouse events to be disabled for content that is blocked
	bindEvents: true,

	// be default blockUI will supress tab navigation from leaving blocking content
	// (if bindEvents is true)
	constrainTabKey: true,

	// fadeIn time in millis; set to 0 to disable fadeIn on block
	fadeIn:  200,

	// fadeOut time in millis; set to 0 to disable fadeOut on unblock
	fadeOut:  400,

	// time in millis to wait before auto-unblocking; set to 0 to disable auto-unblock
	timeout: 0,

	// disable if you don't want to show the overlay
	showOverlay: true,

	// if true, focus will be placed in the first available input field when
	// page blocking
	focusInput: true,

	// suppresses the use of overlay styles on FF/Linux (due to performance issues with opacity)
	applyPlatformOpacityRules: true,
	
	// callback method invoked when fadeIn has completed and blocking message is visible
	onBlock: null,

	// callback method invoked when unblocking has completed; the callback is
	// passed the element that has been unblocked (which is the window object for page
	// blocks) and the options that were passed to the unblock call:
	//	 onUnblock(element, options)
	onUnblock: null,

	// don't ask; if you really must know: http://groups.google.com/group/jquery-en/browse_thread/thread/36640a8730503595/2f6a79a77a78e493#2f6a79a77a78e493
	quirksmodeOffsetHack: 4,

	// class name of the message block
	blockMsgClass: 'blockMsg'
};

// private data and functions follow...

var pageBlock = null;
var pageBlockEls = [];

function install(el, opts) {
	var full = (el == window);
	var msg = opts && opts.message !== undefined ? opts.message : undefined;
	opts = $.extend({}, $.blockUI.defaults, opts || {});
	opts.overlayCSS = $.extend({}, $.blockUI.defaults.overlayCSS, opts.overlayCSS || {});
	var css = $.extend({}, $.blockUI.defaults.css, opts.css || {});
	var themedCSS = $.extend({}, $.blockUI.defaults.themedCSS, opts.themedCSS || {});
	msg = msg === undefined ? opts.message : msg;

	// remove the current block (if there is one)
	if (full && pageBlock)
		remove(window, {fadeOut:0});

	// if an existing element is being used as the blocking content then we capture
	// its current place in the DOM (and current display style) so we can restore
	// it when we unblock
	if (msg && typeof msg != 'string' && (msg.parentNode || msg.jquery)) {
		var node = msg.jquery ? msg[0] : msg;
		var data = {};
		$(el).data('blockUI.history', data);
		data.el = node;
		data.parent = node.parentNode;
		data.display = node.style.display;
		data.position = node.style.position;
		if (data.parent)
			data.parent.removeChild(node);
	}

	$(el).data('blockUI.onUnblock', opts.onUnblock);
	var z = opts.baseZ;

	// blockUI uses 3 layers for blocking, for simplicity they are all used on every platform;
	// layer1 is the iframe layer which is used to supress bleed through of underlying content
	// layer2 is the overlay layer which has opacity and a wait cursor (by default)
	// layer3 is the message content that is displayed while blocking

	var lyr1 = ($.browser.msie || opts.forceIframe) 
		? $('<iframe class="blockUI" style="z-index:'+ (z++) +';display:none;border:none;margin:0;padding:0;position:absolute;width:100%;height:100%;top:0;left:0" src="'+opts.iframeSrc+'"></iframe>')
		: $('<div class="blockUI" style="display:none"></div>');
	
	var lyr2 = opts.theme 
	 	? $('<div class="blockUI blockOverlay ui-widget-overlay" style="z-index:'+ (z++) +';display:none"></div>')
	 	: $('<div class="blockUI blockOverlay" style="z-index:'+ (z++) +';display:none;border:none;margin:0;padding:0;width:100%;height:100%;top:0;left:0"></div>');

	var lyr3, s;
	if (opts.theme && full) {
		s = '<div class="blockUI ' + opts.blockMsgClass + ' blockPage pe-blockui ui-widget ui-corner-all" style="z-index:'+(z+10)+';display:none;position:fixed">' +
			'<div class="ui-widget-content pe-blockui-content"></div></div>';
	}
	else if (opts.theme) {
		s = '<div class="blockUI ' + opts.blockMsgClass + ' blockElement pe-blockui ui-widget ui-corner-all" style="z-index:'+(z+10)+';display:none;position:absolute">' +
			'<div class="ui-widget-content pe-blockui-content"></div></div>';
	}
	else if (full) {
		s = '<div class="blockUI ' + opts.blockMsgClass + ' blockPage" style="z-index:'+(z+10)+';display:none;position:fixed"></div>';
	}			 
	else {
		s = '<div class="blockUI ' + opts.blockMsgClass + ' blockElement" style="z-index:'+(z+10)+';display:none;position:absolute"></div>';
	}
	lyr3 = $(s);

	// if we have a message, style it
	if (msg) {
		if (opts.theme) {
			lyr3.css(themedCSS);
			lyr3.addClass('ui-widget-content');
		}
		else 
			lyr3.css(css);
	}

	// style the overlay
	if (!opts.theme && (!opts.applyPlatformOpacityRules || !($.browser.mozilla && /Linux/.test(navigator.platform))))
		lyr2.css(opts.overlayCSS);
	lyr2.css('position', full ? 'fixed' : 'absolute');

	// make iframe layer transparent in IE
	if ($.browser.msie || opts.forceIframe)
		lyr1.css('opacity',0.0);

	//$([lyr1[0],lyr2[0],lyr3[0]]).appendTo(full ? 'body' : el);
	var layers = [lyr1,lyr2,lyr3], $par = full ? $('body') : $(el);
	$.each(layers, function() {
		this.appendTo($par);
	});
	
	if (opts.theme && opts.draggable && $.fn.draggable) {
		lyr3.draggable({
			handle: '.pe-blockui-titlebar',
			cancel: 'li'
		});
	}

	// ie7 must use absolute positioning in quirks mode and to account for activex issues (when scrolling)
	var expr = setExpr && (!$.boxModel || $('object,embed', full ? null : el).length > 0);
	if (ie6 || expr) {
		// give body 100% height
		if (full && opts.allowBodyStretch && $.boxModel)
			$('html,body').css('height','100%');

		// fix ie6 issue when blocked element has a border width
		if ((ie6 || !$.boxModel) && !full) {
			var t = sz(el,'borderTopWidth'), l = sz(el,'borderLeftWidth');
			var fixT = t ? '(0 - '+t+')' : 0;
			var fixL = l ? '(0 - '+l+')' : 0;
		}

		// simulate fixed position
		$.each([lyr1,lyr2,lyr3], function(i,o) {
			var s = o[0].style;
			s.position = 'absolute';
			if (i < 2) {
				full ? s.setExpression('height','Math.max(document.body.scrollHeight, document.body.offsetHeight) - (jQuery.boxModel?0:'+opts.quirksmodeOffsetHack+') + "px"')
					 : s.setExpression('height','this.parentNode.offsetHeight + "px"');
				full ? s.setExpression('width','jQuery.boxModel && document.documentElement.clientWidth || document.body.clientWidth + "px"')
					 : s.setExpression('width','this.parentNode.offsetWidth + "px"');
				if (fixL) s.setExpression('left', fixL);
				if (fixT) s.setExpression('top', fixT);
			}
			else if (opts.centerY) {
				if (full) s.setExpression('top','(document.documentElement.clientHeight || document.body.clientHeight) / 2 - (this.offsetHeight / 2) + (blah = document.documentElement.scrollTop ? document.documentElement.scrollTop : document.body.scrollTop) + "px"');
				s.marginTop = 0;
			}
			else if (!opts.centerY && full) {
				var top = (opts.css && opts.css.top) ? parseInt(opts.css.top) : 0;
				var expression = '((document.documentElement.scrollTop ? document.documentElement.scrollTop : document.body.scrollTop) + '+top+') + "px"';
				s.setExpression('top',expression);
			}
		});
	}

	// show the message
	if (msg) {
		if (opts.theme)
			lyr3.find('.ui-widget-content').append(msg);
		else
			lyr3.append(msg);
		if (msg.jquery || msg.nodeType)
			$(msg).show();
	}

	if (($.browser.msie || opts.forceIframe) && opts.showOverlay)
		lyr1.show(); // opacity is zero
	if (opts.fadeIn) {
		var cb = opts.onBlock ? opts.onBlock : noOp;
		var cb1 = (opts.showOverlay && !msg) ? cb : noOp;
		var cb2 = msg ? cb : noOp;
		if (opts.showOverlay)
			lyr2._fadeIn(opts.fadeIn, cb1);
		if (msg)
			lyr3._fadeIn(opts.fadeIn, cb2);
	}
	else {
		if (opts.showOverlay)
			lyr2.show();
		if (msg)
			lyr3.show();
		if (opts.onBlock)
			opts.onBlock();
	}

	// bind key and mouse events
	bind(1, el, opts);

	if (full) {
		pageBlock = lyr3[0];
		pageBlockEls = $(':input:enabled:visible',pageBlock);
		if (opts.focusInput)
			setTimeout(focus, 20);
	}
	else
		center(lyr3[0], opts.centerX, opts.centerY);

	if (opts.timeout) {
		// auto-unblock
		var to = setTimeout(function() {
			full ? $.unblockUI(opts) : $(el).unblock(opts);
		}, opts.timeout);
		$(el).data('blockUI.timeout', to);
	}
};

// remove the block
function remove(el, opts) {
	var full = (el == window);
	var $el = $(el);
	var data = $el.data('blockUI.history');
	var to = $el.data('blockUI.timeout');
	if (to) {
		clearTimeout(to);
		$el.removeData('blockUI.timeout');
	}
	opts = $.extend({}, $.blockUI.defaults, opts || {});
	bind(0, el, opts); // unbind events

	if (opts.onUnblock === null) {
		opts.onUnblock = $el.data('blockUI.onUnblock');
		$el.removeData('blockUI.onUnblock');
	}

	var els;
	if (full) // crazy selector to handle odd field errors in ie6/7
		els = $('body').children().filter('.blockUI').add('body > .blockUI');
	else
		els = $('.blockUI', el);

	if (full)
		pageBlock = pageBlockEls = null;

	if (opts.fadeOut) {
		els.fadeOut(opts.fadeOut);
		setTimeout(function() { reset(els,data,opts,el); }, opts.fadeOut);
	}
	else
		reset(els, data, opts, el);
};

// move blocking element back into the DOM where it started
function reset(els,data,opts,el) {
	els.each(function(i,o) {
		// remove via DOM calls so we don't lose event handlers
		if (this.parentNode)
			this.parentNode.removeChild(this);
	});

	if (data && data.el) {
		data.el.style.display = data.display;
		data.el.style.position = data.position;
		if (data.parent)
			data.parent.appendChild(data.el);
		$(el).removeData('blockUI.history');
	}

	if (typeof opts.onUnblock == 'function')
		opts.onUnblock(el,opts);
};

// bind/unbind the handler
function bind(b, el, opts) {
	var full = el == window, $el = $(el);

	// don't bother unbinding if there is nothing to unbind
	if (!b && (full && !pageBlock || !full && !$el.data('blockUI.isBlocked')))
		return;
	if (!full)
		$el.data('blockUI.isBlocked', b);

	// don't bind events when overlay is not in use or if bindEvents is false
	if (!opts.bindEvents || (b && !opts.showOverlay)) 
		return;

	// bind anchors and inputs for mouse and key events
	var events = 'mousedown mouseup keydown keypress';
	b ? $(document).bind(events, opts, handler) : $(document).unbind(events, handler);

// former impl...
//	   var $e = $('a,:input');
//	   b ? $e.bind(events, opts, handler) : $e.unbind(events, handler);
};

// event handler to suppress keyboard/mouse events when blocking
function handler(e) {
	// allow tab navigation (conditionally)
	if (e.keyCode && e.keyCode == 9) {
		if (pageBlock && e.data.constrainTabKey) {
			var els = pageBlockEls;
			var fwd = !e.shiftKey && e.target === els[els.length-1];
			var back = e.shiftKey && e.target === els[0];
			if (fwd || back) {
				setTimeout(function(){focus(back)},10);
				return false;
			}
		}
	}
	var opts = e.data;
	// allow events within the message content
	if ($(e.target).parents('div.' + opts.blockMsgClass).length > 0)
		return true;

	// allow events for content that is not being blocked
	return $(e.target).parents().children().filter('div.blockUI').length == 0;
};

function focus(back) {
	if (!pageBlockEls)
		return;
	var e = pageBlockEls[back===true ? pageBlockEls.length-1 : 0];
	if (e)
		e.focus();
};

function center(el, x, y) {
	var p = el.parentNode, s = el.style;
	var l = ((p.offsetWidth - el.offsetWidth)/2) - sz(p,'borderLeftWidth');
	var t = ((p.offsetHeight - el.offsetHeight)/2) - sz(p,'borderTopWidth');
	if (x) s.left = l > 0 ? (l+'px') : '0';
	if (y) s.top  = t > 0 ? (t+'px') : '0';
};

function sz(el, p) {
	return parseInt($.css(el,p))||0;
};

})(jQuery);
;/**
 * PrimeFaces Extensions BlockUI Widget.
 *
 * @author Oleg Varaksin
 */
PrimeFacesExt.widget.BlockUI = PrimeFaces.widget.BaseWidget.extend({

	/**
	 * Initializes the widget.
	 * 
	 * @param {object} cfg The widget configuration.
	 */
	init : function(cfg) {
		this.sourceId = cfg.source;
		this.targetId = cfg.target;
	    this.contentId = cfg.content;
	    this.contentExtern = cfg.contentExtern;
		this.eventRegEx = cfg.regEx;
		
		// global settings
		$.blockUI.defaults.theme = true;
		$.blockUI.defaults.fadeIn = 0;
		$.blockUI.defaults.fadeOut = 0;
		$.blockUI.defaults.applyPlatformOpacityRules = false;
        
        PrimeFacesExt.removeWidgetScript(cfg.id);
    },
    
	/* public access */
	setupAjaxSend : function () {
        var _self = this;
        $(this.sourceId).ajaxSend(function(event, xhr, ajaxOptions) {
            // first, check if event should be handled 
            if (_self.isAppropriateEvent(ajaxOptions)) {
                _self.block();
            }
        });
	},
		
    setupAjaxComplete : function () {
        var _self = this;
        $(this.sourceId).ajaxComplete(function(event, xhr, ajaxOptions) {
            // first, check if event should be handled
            if (_self.isAppropriateEvent(ajaxOptions)) {
                _self.unblock();
            }
        });
    },
	    
	block : function () {
        var targetEl = $(this.targetId);
        
        // second, check if the target element has been found
        if (targetEl.length > 0) {
            // block the target element
            if (this.contentId != null) {
                if (this.contentExtern) {
                    targetEl.block({message: $(this.contentId).clone().show().wrap('<div>').parent().html()});
                } else {
                    targetEl.block({message: $(this.contentId).html()});
                }
            } else {
                targetEl.block();
            }
    
            // get the current counter
            var blocksCount = targetEl.data("blockUI.blocksCount");
            if (typeof blocksCount === 'undefined') {
                blocksCount = 0;
            }
            
            // increase the counter
            targetEl.data("blockUI.blocksCount", blocksCount+1);
        }        
    },
	    
	unblock : function () {
        var targetEl = $(this.targetId);
        
        // second, check if the target element has been found
        if (targetEl.length > 0) {
            // get the current counter
            var blocksCount = targetEl.data("blockUI.blocksCount");
            
            // check the counter
            if (typeof blocksCount !== 'undefined') {
                if (blocksCount == 1) {
                    // unblock the target element and reset the counter
                    $(this.targetId).unblock();
                    targetEl.data("blockUI.blocksCount", 0);
                } else if (blocksCount > 1) {
                    // only decrease the counter
                    targetEl.data("blockUI.blocksCount", blocksCount-1);
                }
            }
        }
    },
		
	/* private access */
		
	isAppropriateEvent : function (ajaxOptions) {
        if (typeof ajaxOptions === 'undefined' || ajaxOptions == null ||
            typeof ajaxOptions.data === 'undefined' || ajaxOptions.data == null) {
            return false;
        }
        
        // split options around ampersands
        var params = ajaxOptions.data.split(/&/g);
        
        // loop over the ajax options and try to match events
        for (var i = 0; i < params.length; i++) {
            if (this.eventRegEx.test(params[i])) {
                return true;
            }
        }
        
        return false;
    }
});
;/**
 * Resolves the resources for the CKEditor.
 * 
 * @param {string} resource The requested resource from CKEditor.
 * @returns {string} The faces resource URL.
 */
CKEDITOR_GETURL = function(resource) {
	var facesResource;

	//do not resolve
	if (resource.indexOf('?resolve=false') !== -1) {
		facesResource = resource.replace('?resolve=false', '');
	} else {
		//already wrapped?
		var libraryVersionIndex = resource.indexOf('v=' + PrimeFacesExt.getPrimeFacesExtensionsVersion());
		if (libraryVersionIndex !== -1) {
			//look for appended resource
			var appendedResource = resource.substring(libraryVersionIndex + ('v=' + PrimeFacesExt.getPrimeFacesExtensionsVersion()).length);
	
			if (appendedResource.length > 0) {
				//remove append resource from url
				facesResource = resource.substring(0, resource.length - appendedResource.length);
	
				var resourceIdentiferPosition = facesResource.indexOf(PrimeFacesExt.RESOURCE_IDENTIFIER);
				
				if (PrimeFacesExt.isExtensionMapping()) {
					var extensionMappingPosition = facesResource.indexOf(PrimeFacesExt.getRequestUrlExtension());
	
					//extract resource
					var extractedResource = facesResource.substring(resourceIdentiferPosition + PrimeFacesExt.RESOURCE_IDENTIFIER.length, extensionMappingPosition);
	
					facesResource = PrimeFacesExt.getPrimeFacesExtensionsCompressedResource(extractedResource + appendedResource);
				} else {
					var questionMarkPosition = facesResource.indexOf('?');
	
					//extract resource
					var extractedResource = facesResource.substring(resourceIdentiferPosition + PrimeFacesExt.RESOURCE_IDENTIFIER.length, questionMarkPosition);
	
					facesResource = PrimeFacesExt.getPrimeFacesExtensionsCompressedResource(extractedResource + appendedResource);
				}
			} else {
				facesResource = resource;
			}
		} else {
			facesResource = PrimeFacesExt.getPrimeFacesExtensionsCompressedResource('/ckeditor/' + resource);
		}
	}

	return facesResource;
}

/**
 * PrimeFaces Extensions CKEditor Widget.
 * 
 * @author Thomas Andraschko
 */
PrimeFacesExt.widget.CKEditor = PrimeFaces.widget.BaseWidget.extend({
	
	/**
	 * Initializes the widget.
	 * 
	 * @param {object} cfg The widget configuration.
	 */
	init : function(cfg) {
		this._super(cfg);
	
		this.dirtyEventDefined = this.cfg.behaviors && this.cfg.behaviors['dirty'];
		this.changeEventDefined = this.cfg.behaviors && this.cfg.behaviors['change'];
	    this.dirtyState = false;
	
		this.options = {};
		//add widget to ckeditor config, this is required for the save event
		this.options.widget = this;
	
		if (this.cfg.skin) {
			this.options.skin = this.cfg.skin;
		}
		if (this.cfg.width) {
			this.options.width = this.cfg.width;
		}
		if (this.cfg.height) {
			this.options.height = this.cfg.height;
		}
		if (this.cfg.theme) {
			this.options.theme = this.cfg.theme;
		}
		if (this.cfg.toolbar) {
			this.options.toolbar = this.cfg.toolbar;
		}
		if (this.cfg.readOnly) {
			this.options.readOnly = this.cfg.readOnly;
		}
		if (this.cfg.interfaceColor) {
			this.options.uiColor = this.cfg.interfaceColor;
		}
		if (this.cfg.language) {
			this.options.language = this.cfg.language;
		}
		if (this.cfg.defaultLanguage) {
			this.options.defaultLanguage = this.cfg.defaultLanguage;
		}
		if (this.cfg.contentsCss) {
			this.options.contentsCss = this.cfg.contentsCss;
		}
		if (this.cfg.customConfig) {
			this.options.customConfig = this.cfg.customConfig + "?resolve=false";
		}
	
		//check if ckeditor is already included
		if (typeof(CKEDITOR) == 'undefined') {
			var ckEditorScriptURI =
				PrimeFacesExt.getPrimeFacesExtensionsCompressedResource('/ckeditor/ckeditor.js');
			
			var jQueryAdapterScriptURI =
				PrimeFacesExt.getPrimeFacesExtensionsCompressedResource('/ckeditor/adapters/jquery.js');		
	
			//load ckeditor
			PrimeFacesExt.getScript(ckEditorScriptURI, $.proxy(function(data, textStatus) {
	
				//load jquery adapter
				PrimeFacesExt.getScript(jQueryAdapterScriptURI, $.proxy(function(data, textStatus) {
	
					//overwrite save button and init editor
					this.overwriteSaveButton();
					this.initialize();
	
				}, this), true);
	
			}, this), true);
	
		} else {
			this.initialize();
		}
	},

	/**
	 * Initializes the CKEditor instance.
	 * This method will be called when the resources for the CKEditor are loaded.
	 * 
	 * @private
	 */
	initialize : function() {
		//remove old instances if required
		var oldInstance = CKEDITOR.instances[this.id];
		if (oldInstance) {
			oldInstance.destroy(true);
		}
	
		//initialize ckeditor after all resources were loaded
		this.jq.ckeditor($.proxy(function() { this.initialized(); }, this), this.options);
	},

	/**
	 * Overwrites the save button.
	 *
	 * @private
	 */
	overwriteSaveButton : function() {
		//overwrite save button
		CKEDITOR.plugins.registered['save'] = {
			init : function(editor) {
				//get widget
				var widget = editor.config.widget;
				var command = editor.addCommand('save', {
					modes : { wysiwyg:1, source:1 },
					exec : function(editor) {
						if (widget.cfg.behaviors) {
							var saveCallback = widget.cfg.behaviors['save'];
						    if (saveCallback) {
						    	var ext = {
						    			params: []
						    	};
	
						    	saveCallback.call(widget, null, ext);
						    }
						}
					}
				});

				editor.ui.addButton('Save', {label : editor.lang.save, command : 'save' });
			}
		}
	},

	/**
	 * This method will be called when the CKEditor was initialized. 
	 *
	 * @private
	 */
	initialized : function() {
		//get instance
		this.instance = this.jq.ckeditorGet();
		
		//fire initialize event
		this.fireEvent('initialize');
		
		//register blur and focus event
	    this.instance.on('blur', $.proxy(function() { this.handleBlur(); }, this));
	    this.instance.on('focus', $.proxy(function() { this.fireEvent('focus'); }, this));
	
	    //register callbacks for change event
	    if (this.changeEventDefined || this.dirtyEventDefined) {  
	    	
	    	this.bindChangeEventsForWYSIWYGMode();
	    	this.bindCommonChangeEvents();
	
	    	//changes to WYSIWYG mode
	    	this.instance.on('contentDom', $.proxy(function() {
	    		this.bindChangeEventsForWYSIWYGMode();
	    		this.fireEvent('wysiwygMode');
	      	}, this));
	 
	    	//changes to source mode
	    	this.instance.on('mode', $.proxy(function(event) {
				if (this.instance.mode != 'source') {
					return;    	
				}
				this.bindChangeEventsForSourceMode();
				this.fireEvent('sourceMode');
	    	}, this));
	    }
	
	    if (this.dirtyEventDefined && this.cfg.checkDirtyInterval !== 0) {
		    this.dirtyCheckInterval = setInterval($.proxy(this.checkDirtyFromTimer, this), this.cfg.checkDirtyInterval);
	    }
	},

	/**
	 * Binds the common events, which will be used for dirty/change checking.
	 *
	 * @private
	 */
	bindCommonChangeEvents : function() {
		this.instance.on('paste', $.proxy(this.checkDirty, this));
		this.instance.getCommand('undo').on('afterUndo', $.proxy(this.checkDirty, this));
		this.instance.getCommand('redo').on('afterRedo', $.proxy(this.checkDirty, this));
	
		this.instance.on('saveSnapshot', $.proxy(function(event) {
			if (!event.data || !event.data.contentOnly) {
				this.checkDirty();
			}
		}, this));
	
		this.instance.on('afterCommandExec', $.proxy(function(event) {
			if (event.data.name == 'source') {
				return;
			}
	
			if (event.data.command.canUndo !== false) {
				this.checkDirty();
			}
		}, this));
	},

	/**
	 * Binds the events for the WYSIWYG mode, which will be used for dirty/change checking.
	 *
	 * @private
	 */
	bindChangeEventsForWYSIWYGMode : function() {
	    this.instance.document.on('drop', $.proxy(this.checkDirty, this));
	    this.instance.document.getBody().on('drop', $.proxy(this.checkDirty, this));
	    this.instance.document.on('keydown', $.proxy(function(event) {
	    	//do not capture ctrl and meta keys
	    	if (event.data.$.ctrlKey ||event.data.$.metaKey) {
	    		return;
	    	}
	
	    	//filter movement keys and related
	    	var keyCode = event.data.$.keyCode;
	    	if (keyCode == 8 || keyCode == 13 || keyCode == 32
	    			|| ( keyCode >= 46 && keyCode <= 90)
	    			|| ( keyCode >= 96 && keyCode <= 111)
	    			|| ( keyCode >= 186 && keyCode <= 222)) {
	    		this.checkDirty();
	    	}
	    }, this));
	},

	/**
	 * Binds the events for the source mode, which will be used for dirty/change checking.
	 *
	 * @private
	 */
	bindChangeEventsForSourceMode : function() {
		this.instance.textarea.on('drop', $.proxy(this.checkDirty, this));
		this.instance.textarea.on('input', $.proxy(this.checkDirty, this));
		/*
		this.instance.textarea.on('keydown', $.proxy(function(event) {
			//do not capture ctrl and meta keys
			if (!event.data.$.ctrlKey && !event.data.$.metaKey) {
				this.checkDirty();
			}
		}, this));;
		*/
	},

	/**
	 * Sets the dirty state.
	 *
	 * @private
	 */
	checkDirty : function() {
	    if (this.isDirty()) {
	        this.instance.resetDirty();
	
	        if (this.dirtyState === false) {
	        	this.dirtyState = true;
	        }
	        
	        if (this.dirtyEventDefined) {
	        	this.fireEvent('dirty');
	        }
	    }
	},

	/**
	 * Restores the dirtyState and calls check checkDirty();
	 *
	 * @private
	 */
	checkDirtyFromTimer : function() {
		this.dirtyState = false;
		this.checkDirty();
	},

	/**
	 * Handles the blur event and fires the change event if required.
	 *
	 * @author Thomas Andraschko
	 * @private
	 */
	handleBlur : function() {
		this.fireEvent('blur');
	
		if (this.changeEventDefined) {
			if (this.dirtyState) {
			    this.instance.resetDirty();
			    this.fireEvent('change');
			}
	
			this.dirtyState = false;
		}
	},

	/**
	 * This method fires an event if the behavior was defined.
	 *
	 * @param {string} eventName The name of the event.
	 * @private
	 */
	fireEvent : function(eventName) {
		if (this.cfg.behaviors) {
			var callback = this.cfg.behaviors[eventName];
		    if (callback) {
		    	var ext = {
		    			params: []
		    	};
	
		    	callback.call(this, null, ext);
		    }
		}
	},

	/**
	 * Destroys the CKEditor instance.
	 */
	destroy : function() {
		if (this.dirtyCheckInterval) {
			clearInterval(this.dirtyCheckInterval);
		}
	
	    if (this.instance) {
	        this.instance.destroy(true);
	        this.instance = null;
	    }
	
	    this.jq.show();
	},

	/**
	 * Checks if the editor is in dirty state.
	 */
	isDirty : function() {
		return this.dirtyState || this.instance.checkDirty();
	},

	/**
	 * Sets readOnly to the CKEditor.
	 */
	setReadOnly : function(readOnly) {
	    this.instance.setReadOnly(readOnly !== false);
	},

	/**
	 * Checks if the CKEditor is readOnly.
	 */
	isReadOnly : function() {
	    return this.instance.readOnly;
	},

	/**
	 * Indicates that the editor instance has focus.
	 */
	hasFocus : function() {
	    return this.instance.focusManager.hasFocus;
	},

	/**
	 * Returns the CKEditor instance.
	 */
	getEditorInstance : function() {
	    return this.instance;
	}
});
;/**
 * PrimeFaces Extensions DynaForm Widget.
 *
 * @author Oleg Varaksin
 */
PrimeFacesExt.widget.DynaForm = PrimeFaces.widget.BaseWidget.extend({

    /**
     * Initializes the widget.
     *
     * @param {object} cfg The widget configuration.
     */
    init : function(cfg) {
        this._super(cfg);
        if (!cfg.isPostback) {
            this.toggledExtended = false;
        }

        if (cfg.autoSubmit && !window[cfg.widgetVar]) {
            this.submitForm();
        } else if (cfg.isPostback && this.toggledExtended && this.uuid == cfg.uuid) {
            var rows = this.jq.find("tr.pe-dynaform-extendedrow");
            if (rows.length > 0) {
                if (this.openExtended) {
                    rows.show();
                } else {
                    rows.hide();
                }
            }
        }
        
        this.uuid = cfg.uuid;
    },

    toggleExtended : function() {
        var rows = this.jq.find("tr.pe-dynaform-extendedrow");
        if (rows.length > 0) {
            rows.toggle();

            this.toggledExtended = true;
            this.openExtended = $(rows[0]).css("display") != "none";
        }
    },

    submitForm : function() {
        this.jq.find(":submit").trigger('click');
    }
});;/**
 * PrimeFaces Extensions ImageRotateAndResize Widget.
 *
 * @author Thomas Andraschko
 */
PrimeFacesExt.widget.ImageRotateAndResize = PrimeFaces.widget.BaseWidget.extend({
	
	/**
	 * Initializes the widget.
	 * 
	 * @param {object} cfg The widget configuration.
	 */
	init : function(cfg) {
		this.id = cfg.id;
		this.cfg = cfg;

		this.initialized = false;
		
		PrimeFacesExt.removeWidgetScript(this.id);
	},

	/**
	 * Initializes the settings.
	 *
	 * @private
	 */
	initializeLazy : function() {
		if (!this.initialized) {
			this.target = $(this.cfg.target)[0];
			this.imageSrc = this.target.src;
			this.imageWidth = this.target.width;
			this.imageHeight = this.target.height;
			this.degree = 0;
			this.newImageWidth = this.target.width;
			this.newImageHeight = this.target.height;
			this.initialized = true;
		}
	},

	/**
	 * Reloads the widget.
	 */
	reload : function() {
		this.initialized = false;
		this.initializeLazy();
	},

	/**
	 * Rotates the image to the left direction.
	 * 
	 * @param {number} degree The degree.
	 */
	rotateLeft : function(degree) {
		this.initializeLazy();
	
		if ((this.degree - degree) <= 0) {
			this.degree = 360 - ((this.degree - degree) * -1);
		}
		else {
			this.degree -= degree;
		}
	
		this.redrawImage();
		this.fireRotateEvent();
	},

	/**
	 * Rotates the image to the right direction.
	 * 
	 * @param {number} degree The degree.
	 */
	rotateRight : function(degree) {
		this.initializeLazy();
	
		if ((this.degree + degree) >= 360) {
			this.degree = (this.degree + degree) - 360;
		}
		else {
			this.degree += degree;
		}
	
		this.redrawImage();
		this.fireRotateEvent();
	},

	/**
	 * Resizes the image to the given width and height.
	 * 
	 * @param {number} width The new width of the image.
	 * @param {number} height The new height of the image.
	 */
	resize : function(width, height) {
		this.initializeLazy();
	
		this.newImageWidth = width;
		this.newImageHeight = height;
	
		this.redrawImage();
		this.fireResizeEvent();
	},

	/**
	 * Scales the image with the given factor.
	 * 
	 * @param {number} scaleFactor The scale factor. For example: 1.1 = scales the image to 110% size.
	 */
	scale : function(scaleFactor) {
		this.initializeLazy();
	
		this.newImageWidth = this.newImageWidth * scaleFactor;
		this.newImageHeight = this.newImageHeight * scaleFactor;
	
		this.redrawImage();
		this.fireResizeEvent();
	},

	/**
	 * Restores the default image.
	 * It also fires the rotate and resize event with the default values.
	 */
	restoreDefaults : function() {
		this.initializeLazy();
	
		this.newImageWidth = this.imageWidth;
		this.newImageHeight = this.imageHeight;
		this.degree = 0;
	
		this.redrawImage();
		this.fireResizeEvent();
		this.fireRotateEvent();
	},

	/**
	 * Redraws the image.
	 *
	 * @private
	 */
	redrawImage : function() {
		var rotation;
		if (this.degree >= 0) {
			rotation = Math.PI * this.degree / 180;
		} else {
			rotation = Math.PI * (360 + this.degree) / 180;
		}
	
		var cos = Math.cos(rotation);
		var sin = Math.sin(rotation);
	
		//check for < IE9, otherwise use canvas
		if ($.browser.msie && parseInt($.browser.version) <= 8) {
			//create new image
			var image = document.createElement('img');
			image.src = this.imageSrc;
			
			//set new size
			image.height = this.newImageHeight;
			image.width = this.newImageWidth;
	
			//apply rotation for IE
			image.style.filter = "progid:DXImageTransform.Microsoft.Matrix(M11=" + cos + ",M12=" + (sin * -1) + ",M21=" + sin + ",M22=" + cos + ",SizingMethod='auto expand')";	
			
			//replace old image with new generated one
			image.id = this.target.id;
			this.target.parentNode.replaceChild(image, this.target);
			this.target = image;
		} else {
			//create canvas instead of img
			var canvas = document.createElement('canvas');
	
			//new image with new size
			var newImage = new Image();
			newImage.src = this.imageSrc;
			newImage.width = this.newImageWidth;
			newImage.height = this.newImageHeight;
	
			//rotate
			canvas.style.width = canvas.width = Math.abs(cos * newImage.width) + Math.abs(sin * newImage.height);
			canvas.style.height = canvas.height = Math.abs(cos * newImage.height) + Math.abs(sin * newImage.width);
	
			var context = canvas.getContext('2d');
			context.save();
			
			if (rotation <= Math.PI/2) {
				context.translate(sin * newImage.height, 0);
			} else if (rotation <= Math.PI) {
				context.translate(canvas.width, (cos * -1) * newImage.height);
			} else if (rotation <= 1.5 * Math.PI) {
				context.translate((cos * -1) * newImage.width, canvas.height);
			} else {
				context.translate(0, (sin * -1) * newImage.width);
			}
	
			context.rotate(rotation);
			context.drawImage(newImage, 0, 0, newImage.width, newImage.height);
			context.restore();	
	
			//replace image with canvas and set src attribute
			canvas.id = this.target.id;
			canvas.src = this.target.src;
			this.target.parentNode.replaceChild(canvas, this.target);
			this.target = canvas;
		}
	},

	/**
	 * Fires the rotate event.
	 *
	 * @private
	 */
	fireRotateEvent : function() {
		if (this.cfg.behaviors) {
			var callback = this.cfg.behaviors['rotate'];
		    if (callback) {
		    	var ext = {
		    			params: [
		    			         { name: this.id + '_degree', value: this.degree }
		    			]
		    	};

		    	callback.call(this, null, ext);
		    }
		}
	},

	/**
	 * Fires the resize event.
	 *
	 * @private
	 */
	fireResizeEvent : function() {
		if (this.cfg.behaviors) {
			var callback = this.cfg.behaviors['resize'];
		    if (callback) {
		    	var ext = {
		    			params: [
		    			         { name: this.id + '_width', value: this.newImageWidth },
		    			         { name: this.id + '_height', value: this.newImageHeight }
		    			]
		    	};

		    	callback.call(this, null, ext);
		    }
		}
	}
});
;/**
 * PrimeFaces Extensions TriStateCheckbox Widget.
 *
 * @author Mauricio Fenoglio
 */
PrimeFacesExt.widget.TriStateCheckbox = PrimeFaces.widget.BaseWidget.extend({

	/**
	 * Initializes the widget.
	 * 
	 * @param {object} cfg The widget configuration.
	 */
	init : function(cfg) {
	    this._super(cfg);

	    this.input = $(this.jqId + '_input');
	    this.box = this.jq.find('.ui-chkbox-box');
	    this.icon = this.box.children('.ui-chkbox-icon');
	    this.itemLabel = this.jq.find('.ui-chkbox-label');
	    this.disabled = this.input.is(':disabled');
	    
	    var _self = this;
	
	    //bind events if not disabled
	    if (!this.disabled) {
	        this.box.mouseover(function() {
	            _self.box.addClass('ui-state-hover');
	        }).mouseout(function() {
	            _self.box.removeClass('ui-state-hover');
	        }).click(function() {
	            _self.toggle();
	        });
	        
	        //toggle state on label click
	        this.itemLabel.click(function() {
	            _self.toggle();
	        });
	        
	        //Client Behaviors
	        if(this.cfg.behaviors) {
	            PrimeFaces.attachBehaviors(this.input, this.cfg.behaviors);
	        }
	    }
        
        // pfs metadata
        this.input.data(PrimeFaces.CLIENT_ID_DATA, this.id);
	},

	toggle : function() {   
	    if (!this.disabled) {
	        var value = (this.input.attr('value')+1) % 3;        
	        this.input.attr('value', value); 
	        this.changeIcon(value);
	        this.input.change();
	    }
	},

	changeIcon : function(addIndex) {
	    if (!this.disabled) {               
	        iconsClasses =  this.box.attr('statesicons').split(';');

	        //js bug for mod of negative values. fix with this!!!!!!!
	        removeIndex = (((addIndex-1)%3)+3)%3;      
	        removeClass = 'ui-icon ' + iconsClasses[removeIndex];
	        if(iconsClasses[addIndex]!=" ") {
	            addClass = 'ui-icon ' + iconsClasses[addIndex];
	        } else {
	            addClass = '';
	        }

	        //if addIndex is 0 , remove active class
	        if (addIndex==0) {
	            this.box.removeClass('ui-state-active');
	        } else{
	            this.box.addClass('ui-state-active')
	        }

	        //remove old icon and add the new one        
	        this.box.children('.ui-chkbox-icon').removeClass(removeClass);
	        this.box.children('.ui-chkbox-icon').addClass(addClass);       
	    }
	}
});
;/**
 * PrimeFaces Extensions TriStateManyCheckbox Widget.
 *
 * @author Mauricio Fenoglio
 */
PrimeFacesExt.widget.TriStateManyCheckbox = PrimeFaces.widget.BaseWidget.extend({

	/**
	 * Initializes the widget.
	 * 
	 * @param {object} cfg The widget configuration.
	 */
	init : function(cfg) {
	    this._super(cfg);

	    this.outputs = this.jq.find('.ui-chkbox-box:not(.ui-state-disabled)');
	    this.inputs = this.jq.find(':text:not(:disabled)');
	    this.labels = this.jq.find('label:not(.ui-state-disabled)');
	    var _self = this;
	
	    this.outputs.mouseover(function() {
	        $(this).addClass('ui-state-hover');
	    }).mouseout(function() {
	        $(this).removeClass('ui-state-hover');
	    }).click(function() {
	        _self.toggle($(this));
	    });
	
	    this.labels.click(function(e) {
	        var element = $(this),
	        input = $(PrimeFaces.escapeClientId(element.attr('for'))),
	        checkbox = input.parent().next();
	        checkbox.click();
	        
	        e.preventDefault();
	    });
	
	    //Client Behaviors
	    if (this.cfg.behaviors) {
	        PrimeFaces.attachBehaviors(this.inputs, this.cfg.behaviors);
	    }
        
        // pfs metadata
        this.inputs.data(PrimeFaces.CLIENT_ID_DATA, this.id);
	},
	
	toggle : function(checkbox) {
	    var inputField = checkbox.prev().find(':input');   
	    if (!checkbox.hasClass('ui-state-disabled')) {
	        var value = (inputField.attr('value')+1) % 3;        
	        inputField.attr('value', value); 
	        this.changeIcon(checkbox,value);
	        inputField.change();
	    }
	},

	changeIcon : function(checkbox, addIndex) {
        iconsClasses =  checkbox.attr('statesicons').split(';');
        //js bug for mod of negative values. fix with this!!!!!!!
        removeIndex = (((addIndex-1)%3)+3)%3;      
        removeClass = 'ui-icon ' + iconsClasses[removeIndex];
        if (iconsClasses[addIndex]!=" ") {
            addClass = 'ui-icon ' + iconsClasses[addIndex];
        } else{
            addClass = '';
        }

        //if addIndex is 0 , remove active class
        if (addIndex==0) {
            checkbox.removeClass('ui-state-active');
        } else {
            checkbox.addClass('ui-state-active')
        }
        //remove old icon and add the new one        
        checkbox.children('.ui-chkbox-icon').removeClass(removeClass);
        checkbox.children('.ui-chkbox-icon').addClass(addClass);          
	}
});
;/**
 * PrimeFaces Extensions Timeline Widget.
 *
 * @author Nilesh Namdeo Mali
 */
PrimeFacesExt.widget.Timeline = PrimeFaces.widget.BaseWidget.extend({
    
        /**
	 * Initializes the widget.
	 * 
	 * @param {object} cfg The widget configuration.
	 */
	init : function(cfg) {
	    this._super(cfg);
	    
	    this.jq.append('<div id="'+ this.id + '_main" class="pe-timeline ui-widget-content ui-corner-all"><div id="'+ this.id + '_scroll"></div></div>');
	    this.container = $('#' + this.id + '_scroll');    
	    
	    if(this.cfg.dataSource) {
	        this.bindTimelines();
	    }
	},

	bindTimelines : function() {
	    var dataSource = this.cfg.dataSource;    
	    var size = dataSource.length;
	    var timelineWidth = 320 * size;     
	    this.container.width(timelineWidth + 10);
	    
	    $('#' + this.id + '_main').mousedown(function (event) {
	        $(this)
	        .data('down', true)
	        .data('x', event.clientX)
	        .data('scrollLeft', this.scrollLeft);
	                
	        return false;
	    }).mouseup(function (event) {
	        $(this).data('down', false);
	    }).mousemove(function (event) {
	        if ($(this).data('down') == true) {
	            this.scrollLeft = $(this).data('scrollLeft') + $(this).data('x') - event.clientX;
	        }
	    }).mousewheel(function (event, delta) {
	        this.scrollLeft -= (delta * 30);
	    }).css({
	        'overflow' : 'hidden',
	        'cursor' : '-moz-grab'
	    });
	    
	    for(var index = 0; index < size; index++){    
	        this.createTimeline(dataSource[index]);
	    }    
	},

	createTimeline : function(tl) {
	    var dom = '<div id="' + tl.id + '" class="pe-timeline-menu">';
	    dom += '<div class="pe-timeline-menu-header ui-widget-header ui-corner-all">';
	    dom += tl.title + '</div><ul id="'+ tl.id +'_ul" class="pe-timeline-event-list"></ul></div>';    
	    this.container.append(dom);
	    var eventContainer = $('#' + tl.id + '_ul');
	    var size = tl.events.length;
	    for(var index = 0; index < size; index++){
	        this.createTimelineEvent(tl.events[index], eventContainer);        
	    }        
	},

	createTimelineEvent : function(tlEvent, container) {
	    var eventId = tlEvent.id;    
	    var dom = '<li id="' + eventId + '_li" class="ui-widget-content ui-corner-all"><span></span>';    
	    dom += tlEvent.title;
	    dom += '<div id="'+ eventId + '_content" class="content">';
	    dom += tlEvent.startDate + ((tlEvent.startDate == '' || tlEvent.endDate == '')?'':' - ') + tlEvent.endDate + '<br/>' + tlEvent.description;
	    dom += '</div><li>';    
	    container.append(dom);
	    
	    // Hover for TimelineEvent
	    $('#' + eventId + '_li').hover(function () {
	        $(this).addClass("ui-state-hover");
	    }, function () {
	        $(this).removeClass("ui-state-hover");
	    });    
	    var targetId = this.id;
	    
	    // BlockUI on click
	    $('#' + eventId + '_li').click(function(){
	        var blockUI = new PrimeFacesExt.widget.BlockUI({
                    id: eventId + '_jsf',    
	            source: '#' + eventId,
	            target: '#' + targetId,
	            content: '#' + eventId + '_content'
	        });
	        blockUI.block();
	        $('.ui-widget-overlay').attr("title","Click to unblock").click(function(){                        
                        blockUI.unblock();
                });
	    });
	}
});
